<?php

/*
Plugin Name: Stocksy Image Displayer
Plugin URI:  http://seanlockephotography.com/extras 
Description: Adds a widget to display Stocksy images
Version:     1.1
Author:      Sean Locke
Author URI:  http://seanlockephotography.com
License:     GPL2 etc
License URI: 

Copyright 2018 Sean Locke ( email : sjlocke@yahoo.com )
(Plugin Name) is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 2 of the License, or
any later version.

*/

// The widget class
class sid_stocksyImageDisplayer extends WP_Widget {

	// Main constructor
	public function __construct() {
	$widget_ops = array( 
			'classname' => 'sid_stocksyImageDisplayer',
			'description' => 'A plugin to display Stocksy Images',
			'customize_selective_refresh' => true
		);
		parent::__construct( 'sid_stocksyImageDisplayer', 'Stocksy Image Display', $widget_ops );
		
		// this is how we make sure the masonry javascript is available to use later
		add_action('wp_enqueue_scripts', function(){wp_enqueue_script('jquery-masonry');});
		
		/* ... */
	}

	// The widget form (for the backend )
	public function form( $instance ) {	
	
	// Set widget defaults
	$defaults = array(
		'widgetTitle'    => 'My Stocksy Images',
		'apiKey'     => '',
		'searchString' => '',
		'imageMaxWH' => '150',
		'numImages' => '1',
		'numRows' => '1',
		'numCols' => '1',
		'cellMaxW' => '200',
		'sort' => 'latest',
		'imageTitleLength' => '100',
		'showTitle' => 'on',
		'tableOrMasonry' => 'table'
	);
	
	// Parse current settings with defaults
	extract( wp_parse_args( ( array ) $instance, $defaults ) ); 

	// we turn on/off options depending on whether "table" or "masonry" is chosen
	// here, we're setting the intital value for the display in the admin section
	$tableVisibility = "block";
	$masonryVisibility = "none";
	
	if ($tableOrMasonry == "masonry"){
		$tableVisibility = "none";
		$masonryVisibility = "block";
	}
	
	?>
	
	<?php // Widget Title ?>
	<p>
		<label for="<?php echo esc_attr( $this->get_field_id( 'widgetTitle' ) ); ?>"><?php _e( 'Title:', 'text_domain' ); ?></label>
		<input class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'widgetTitle' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'widgetTitle' ) ); ?>" type="text" value="<?php echo esc_attr( $widgetTitle ); ?>" />
	</p>

	<?php // API Key ?>
	<p>
		<label for="<?php echo esc_attr( $this->get_field_id( 'apiKey' ) ); ?>"><?php _e( 'API Key:', 'text_domain' ); ?></label>
		<br>Get your key from Stocksy Support.
		<input class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'apiKey' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'apiKey' ) ); ?>" type="text" value="<?php echo esc_attr( $apiKey ); ?>" />
	</p>

	<?php // Search String ?>
	<p>
		<label for="<?php echo esc_attr( $this->get_field_id( 'searchString' ) ); ?>"><?php _e( 'Search String:', 'text_domain' ); ?></label>
		<br>Example: "black+dog" or "%22sean+locke%22"
		<input class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'searchString' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'searchString' ) ); ?>" type="text" value="<?php echo esc_attr( $searchString ); ?>" />
	</p>

	<?php // Sort Method ?>
	<p>
		<label for="<?php echo $this->get_field_id( 'sort' ); ?>"><?php _e( 'Sort Method', 'text_domain' ); ?></label>
		<select name="<?php echo $this->get_field_name( 'sort' ); ?>" id="<?php echo $this->get_field_id( 'sort' ); ?>" class="widefat">
		<?php
		// Your options array
		$options = array(
			'latest'        => __( 'Latest', 'text_domain' ),
			'curated' => __( 'Curated', 'text_domain' ),
			'popular' => __( 'Popular', 'text_domain' ),
			'emerging' => __( 'Emerging', 'text_domain' ),
		);

		// Loop through options and add each one to the select dropdown
		foreach ( $options as $key => $name ) {
			echo '<option value="' . esc_attr( $key ) . '" id="' . esc_attr( $key ) . '" '. selected( $sort, $key, false ) . '>'. $name . '</option>';

		} ?>
		</select>
	</p>
	
	<?php // Image Max WH ?>
	<p>
		<label for="<?php echo esc_attr( $this->get_field_id( 'imageMaxWH' ) ); ?>"><?php _e( 'Image Maximum Width Or Height:', 'text_domain' ); ?></label>
		<br>Table layout: image will be scaled equally so largest side is this number. Set to "0" to use original thumbnail dimensions.
		<br>Masonry layout: image will be scaled so width is this number for all images.
		<input name="<?php echo $this->get_field_name( 'imageMaxWH' ); ?>" id="<?php echo $this->get_field_id( 'imageMaxWH' ); ?>" class="widefat" step="1" min="0" value="<?php echo $imageMaxWH ?>" size="3" max="1000" type="number">
	</p>

	<?php // Cell Max WH ?>
	<p>
		<label for="<?php echo esc_attr( $this->get_field_id( 'cellMaxW' ) ); ?>"><?php _e( 'Container Maximum Width:', 'text_domain' ); ?></label>
		<br>Table layout: cells will attempt to be this width (should be bigger than Image Max). Set to "0" to use original thumbnail dimensions.
		<br>Masonry layout: column with gutter (margin) will attempt to be this size.
		<input name="<?php echo $this->get_field_name( 'cellMaxW' ); ?>" id="<?php echo $this->get_field_id( 'cellMaxW' ); ?>" class="widefat" step="1" min="0" value="<?php echo $cellMaxW ?>" size="3" max="1000" type="number">
	</p>
	
	<?php // Table Or Masonry  ?>
	<p>
		<label for="<?php echo $this->get_field_id( 'tableOrMasonry' ); ?>"><?php _e( 'Table Or Masonry Layout', 'text_domain' ); ?></label>
		<select name="<?php echo $this->get_field_name( 'tableOrMasonry' ); ?>" id="<?php echo $this->get_field_id( 'tableOrMasonry' ); ?>" class="widefat">
		<?php
			// Your options array
		$options = array(
			'table' => __( 'table', 'text_domain' ),
			'masonry' => __( 'masonry', 'text_domain' ),
		);
		// Loop through options and add each one to the select dropdown
		foreach ( $options as $key => $name ) {
			echo '<option value="' . esc_attr( $key ) . '" id="' . esc_attr( $key ) . '" '. selected( $tableOrMasonry, $key, false ) . '>'. $name . '</option>';
		}
		?>
		</select>
	</p>
	<?php // Here's where we add the javascript logic to toggle controls on or off in the admin panel, using jQuery  ?>
	<script>
	$(function() {
		$('#<?php echo $this->get_field_id( 'tableOrMasonry' ); ?>').change(function(){
			if($('#<?php echo $this->get_field_id( 'tableOrMasonry' ); ?>').val() == 'table') {
				$('.sid_tableOptions').show(); 
				$('.sid_masonryOptions').hide(); 
			} else {
				$('.sid_tableOptions').hide(); 
				$('.sid_masonryOptions').show(); 
			} 
		});
	});
	</script>

	<?php // Number of Table Rows : set the display initially to hidden or visible and give a class to the p element to be able to grab and toggle  ?>
	<p class = "sid_tableOptions" style="display: <?php echo $tableVisibility ?>">
		<label  for="<?php echo $this->get_field_id( 'numRows' ); ?>"><?php _e( 'Number of Rows', 'text_domain' ); ?></label>
		<input name="<?php echo $this->get_field_name( 'numRows' ); ?>" id="<?php echo $this->get_field_id( 'numRows' ); ?>" class="widefat" step="1" min="1" value="<?php echo $numRows ?>" size="3" max="20" type="number">
	</p>

	<?php // Number of Table Bolumns : set the display initially to hidden or visible and give a class to the p element to be able to grab and toggle  ?>
	<p class = "sid_tableOptions" style="display: <?php echo $tableVisibility ?>">
		<label  for="<?php echo $this->get_field_id( 'numCols' ); ?>"><?php _e( 'Number of Columns', 'text_domain' ); ?></label>
		<input name="<?php echo $this->get_field_name( 'numCols' ); ?>" id="<?php echo $this->get_field_id( 'numCols' ); ?>" class="widefat" step="1" min="1" value="<?php echo $numCols ?>" size="3" max="20" type="number">
	</p>

	<?php // Number of Images To Display : set the display initially to hidden or visible and give a class to the p element to be able to grab and toggle  ?>
	<p class = "sid_masonryOptions" style="display: <?php echo $masonryVisibility ?>">
		<label  for="<?php echo $this->get_field_id( 'numImages' ); ?>"><?php _e( 'Number of Images to Load', 'text_domain' ); ?></label>
		<input name="<?php echo $this->get_field_name( 'numImages' ); ?>" id="<?php echo $this->get_field_id( 'numImages' ); ?>" class="widefat" step="1" min="1" value="<?php echo $numImages ?>" size="3" max="200" type="number">
	</p>
	
	<?php // Show Image Title : the title goes under the image  ?>
	<p>
		<label for="<?php echo $this->get_field_id( 'showTitle' ); ?>"><?php _e( 'Show Image Titles', 'text_domain' ); ?></label>
		<input class="checkbox" type="checkbox" <?php checked( $instance[ 'showTitle' ], 'on' ); ?> id="<?php echo $this->get_field_id( 'showTitle' ); ?>" name="<?php echo $this->get_field_name( 'showTitle' ); ?>" /> 
	</p>

	<?php // Image Title Length  ?>
	<p>
		<label for="<?php echo esc_attr( $this->get_field_id( 'imageTitleLength' ) ); ?>"><?php _e( 'Image Title Text Length', 'text_domain' ); ?></label>
		<br>Letter count of image title before being trucated with "...".		
		<input name="<?php echo $this->get_field_name( 'imageTitleLength' ); ?>" id="<?php echo $this->get_field_id( 'imageTitleLength' ); ?>" class="widefat" step="1" min="5" value="<?php echo $imageTitleLength ?>" size="3" max="500" type="number">
	</p>
	<?php
	}

	// Update widget settings
	public function update( $new_instance, $old_instance ) {
		$instance = $old_instance;
		$instance['widgetTitle']    = isset( $new_instance['widgetTitle'] ) ? wp_strip_all_tags( $new_instance['widgetTitle'] ) : '';
		$instance['apiKey']     = isset( $new_instance['apiKey'] ) ? wp_strip_all_tags( $new_instance['apiKey'] ) : '';
		$instance['searchString']     = isset( $new_instance['searchString'] ) ? wp_strip_all_tags( $new_instance['searchString'] ) : '';
		$instance['sort']   = isset( $new_instance['sort'] ) ? wp_strip_all_tags( $new_instance['sort'] ) : '';
		$instance['numCols']   = isset( $new_instance['numCols'] ) ? wp_strip_all_tags( $new_instance['numCols'] ) : '';
		$instance['numRows']   = isset( $new_instance['numRows'] ) ? wp_strip_all_tags( $new_instance['numRows'] ) : '';
		$instance['showTitle']   = isset( $new_instance['showTitle'] ) ? wp_strip_all_tags( $new_instance['showTitle'] ) : '';
		$instance['numImages']   = isset( $new_instance['numImages'] ) ? wp_strip_all_tags( $new_instance['numImages'] ) : '';
		$instance['tableOrMasonry']   = isset( $new_instance['tableOrMasonry'] ) ? wp_strip_all_tags( $new_instance['tableOrMasonry'] ) : '';
		$instance['imageMaxWH']   = isset( $new_instance['imageMaxWH'] ) ? wp_strip_all_tags( $new_instance['imageMaxWH'] ) : '';
		$instance['imageTitleLength']   = isset( $new_instance['imageTitleLength'] ) ? wp_strip_all_tags( $new_instance['imageTitleLength'] ) : '';
		if (!ctype_digit($instance['imageMaxWH'])) $instance['imageMaxWH'] = 150;
		$instance['cellMaxW']   = isset( $new_instance['cellMaxW'] ) ? wp_strip_all_tags( $new_instance['cellMaxW'] ) : '';
		if (!ctype_digit($instance['cellMaxW'])) $instance['cellMaxW'] = 200;

		return $instance;
}

	// Display the widget
	public function widget( $args, $instance ) {

		// if there aren't any values, set defaults
		$title     = isset( $instance['widgetTitle'] ) ? $instance['widgetTitle'] : '';
		$apiKey     = isset( $instance['apiKey'] ) ? $instance['apiKey'] : '';
		$searchString    = isset( $instance['searchString'] ) ? $instance['searchString'] : '';
		$tableOrMasonry    = isset( $instance['tableOrMasonry'] ) ? $instance['tableOrMasonry'] : 'table';
		$numCols    = isset( $instance['numCols'] ) ? $instance['numCols'] : '1';
		$numRows    = isset( $instance['numRows'] ) ? $instance['numRows'] : '1';
		$numImages    = isset( $instance['numImages'] ) ? $instance['numImages'] : '1';
		$sort = isset( $instance['sort'] ) ? $instance['sort'] : 'latest';
		$imageMaxWH = isset( $instance['imageMaxWH'] ) ? $instance['imageMaxWH'] : '150';
		$tableCellMaxW = isset( $instance['cellMaxW'] ) ? $instance['cellMaxW'] : '200';
		$showTitle = isset( $instance['showTitle'] ) ? $instance['showTitle'] : 'on';
		$maximageTitleLength = isset( $instance['imageTitleLength'] ) ? $instance['imageTitleLength'] : '100';
		
		$displayAmount = 0;
		
		// set how many images to display
		if ($tableOrMasonry == "table") $displayAmount = $numCols * $numRows; else $displayAmount = $numImages;
		
		// begin construction of the search URL
		$searchURL = "https://api.stocksy.com/search?query=" . $searchString . "&limit=" . $displayAmount ."&sort=" . $sort;

		$error = 0;
		// Let's get the API search return
		// step1 - intiate the session
		$cSession = curl_init(); 
		// step2
		// Lets pass the API key through the headers
		$request_headers[] = 'x-api-key: '.$apiKey;
		curl_setopt($cSession,CURLOPT_URL,$searchURL);
		curl_setopt($cSession,CURLOPT_RETURNTRANSFER,true);
		curl_setopt($cSession,CURLOPT_HTTPHEADER, $request_headers); 
		// Turns off HTTPS - SSL
		curl_setopt($cSession, CURLOPT_SSL_VERIFYPEER, false);
		// step3 - run and save in $result
		$result=curl_exec($cSession);
		// step4 - error check
		$error = curl_errno($cSession);	
		// step5 - close session after the error check
		curl_close($cSession);
		
		// WordPress core before_widget hook (always include )
		echo $before_widget;

		// Display the widget
		echo '<div class="stocksyImageDisplayer widget-text wp_widget_plugin_box">';
		echo "<h2 class = \"stocksyImageDisplayer widget-title\">" . $title ."</h2>\n";
		
		// If there was an error, just print a warning, otherwise output the content
		if ($error != 0)
			echo "<p>Unable to retrieve image data.";
		else {
			// Need true as second argument to return an array
			$obj = json_decode($result, true);

			if ($tableOrMasonry == "table") {
				echo "\n<table class = \"sid_table\" style = \"margin: 0 auto;\">";
			
				$currentColumn = 1;
				
				foreach($obj['response']['items'] as $item) {
					
					$myWidth = 0 ;
					$myHeight = 0;
					$myWidth = $item['thumbnail_dimensions']['width'];
					$myHeight = $item['thumbnail_dimensions']['height'];
					$ratio = $myWidth/$myHeight;
					$myCellMaxW = $tableCellMaxW;
					
					if ($imageMaxWH == "0") {	
					}
					else if ($ratio > 1 ) {
						$myWidth = $imageMaxWH;
						$myHeight = $imageMaxWH/$ratio;
					} else {
						$myHeight = $imageMaxWH;
						$myWidth = $imageMaxWH*$ratio;
					}
				
					if ($tableCellMaxW == "0")
						$myCellMaxW = $myWidth;
					
					if ( $currentColumn == 1) 
						echo "\n<tr>";
					
					echo "\n<td width = \"" . ($myCellMaxW) . "px\" style = \"text-align: center; vertical-align: middle; max-width: ". ($myCellMaxW) . "px\" >\n";
					$title = $item['title'];
					echo "<div style = \"width: ". ($myCellMaxW) . "px; margin:auto\"><a href = \"http://stocksy.com/" . $item['id'] . "\"><img src = \"" . $item['thumbUrl'] . "\" title = \"" . $title . "\" width = " . $myWidth . " height = " . $myHeight . "></img></a></div>\n";
					if (strlen($title) > $maximageTitleLength ) $title = substr($title,0,$maximageTitleLength) . "...";
					if($showTitle == "on")
						echo "<div style = \"width: ". ($myCellMaxW) . "px; margin:auto\"><a href = \"http://stocksy.com/" . $item['id'] . "\">".$title."</a></div>\n";
					echo "</td>";
									
					$currentColumn++;
					if ($currentColumn > $numCols) {
						echo "\n</tr>";
						$currentColumn = 1;
					}
				}
						
				// finish off the table
				echo "</table>\n";
			} else {
				// for masonry, we have to set an item width for all the images.  It can't be a per item thing.
				
				if ($imageMaxWH == 0) $imageMaxWH = 100;
				if ($tableCellMaxW ==0) $tableCellMaxW = 105;
				
				?>
				<style>
				.sid_container {margin:0 auto;}
				.sid_item {float: left; margin-bottom:<?php echo (2*($tableCellMaxW - $imageMaxWH)) ?>px; width:}
				.sid_title {width: <?php echo $imageMaxWH ?>px; text-align:center;}
				</style>
				<?php
				echo "\n<div class=\"sid_container\">";
				
				foreach($obj['response']['items'] as $item) {
					
					$myWidth = $item['thumbnail_dimensions']['width'];
					$myHeight = $item['thumbnail_dimensions']['height'];
					$ratio = $myWidth/$myHeight;

					$myWidth = $imageMaxWH;
					$myHeight = $imageMaxWH/$ratio;
				
					echo "\n<div class = \"sid_item\">\n";
					echo "<div style = \"text-align: center\"><a href = \"http://stocksy.com/" . $item['id'] . "\" >";
					// need to add the display:block below to remove vertical gaps
					$title = $item['title'];
					echo "<img src = \"" . $item['thumbUrl'] . "\" width = " . $myWidth . "px height = " . $myHeight . "px style=\"display:block\" title = \"" . $title . "\"></img></a></div>\n";
					if (strlen($title) > $maximageTitleLength ) $title = substr($title,0,$maximageTitleLength) . "...";
					if($showTitle == "on")
						echo "<div class = \"sid_title\"><a href = \"http://stocksy.com/" . $item['id'] . "\">".$title."</a></div>\n";
					echo "</div>";
				}
				
				echo "\n</div>";
				?>

<script>

jQuery(document).ready(function($){ 
	$('.sid_container').imagesLoaded (function () {
		$('.sid_container').masonry({
		// options
			itemSelector: '.sid_item',
			gutter: <?php echo ($tableCellMaxW - $imageMaxWH) ?>,
			columnWidth: <?php echo $tableCellMaxW ?>,
			isFitWidth: true,
			});
		});
	});
</script>
				<?php
				
			}
			
		}
		
		echo '</div>';

		// WordPress core after_widget hook (always include )
		echo $after_widget;
	
	}

}

// Register the widget
function sid_registerWidget() {
	register_widget( 'sid_stocksyImageDisplayer' );
}
add_action( 'widgets_init', 'sid_registerWidget' );



?>